@@ -12,11 +12,11 @@ use Memoize;
 
 =head1 VERSION
 
-Version 1.402.6
+Version 1.402.9
 
 =cut
 
-our $VERSION = '1.402.6';
+our $VERSION = '1.402.9';
 
 my %typeregistry = (
     default => {},
@@ -107,6 +107,7 @@ be found (this could be caused by updating the db).
 =cut
 
 sub clear_info_cache {
+    local ($@);
     eval { Memoize::flush_cache('function_info') };
 }
 
@@ -203,7 +204,7 @@ sub function_info {
     }
 
     my $sth = $dbh->prepare($query) || die $!;
-    $sth->execute(@queryargs);
+    $sth->execute(@queryargs) || die $dbh->errstr . ": " . $query;
     my $ref = $sth->fetchrow_hashref('NAME_lc');
     croak "transaction already aborted" if  $dbh->state eq '25P02';
     croak "No such function" if !$ref;
@@ -276,6 +277,7 @@ the framework level for this parameter.
 sub call_procedure {
     my ($self) = shift @_;
     my %args = @_;
+    local $@;
     $args{funcschema} ||= 'public';
     $args{funcprefix} ||= '';
     $args{funcname} = $args{funcprefix}.$args{funcname};
@@ -297,13 +299,15 @@ sub call_procedure {
 
     my @qargs = map { 
                       my $arg = $_;
+                      local ($@);
                       $arg = $arg->to_db if eval {$arg->can('to_db')};
                       $arg = $arg->pgobject_to_db if eval {$arg->can('pgobject_to_db')};
-                      ref $arg ? $arg->{value} : $arg 
+                      no warnings 'uninitialized';
+                      ref($arg) =~ /HASH/ ? $arg->{value} : $arg;
                 }  @{$args{args}};
 
     my $argstr = join ', ', map { 
-                  (ref $_ and $_->{cast}) ? "?::$_->{cast}" : '?';
+                  (ref $_ and eval { $_->{cast} } ) ? "?::$_->{cast}" : '?';
                   } @{$args{args}};  
 
     my $order = '';
@@ -333,14 +337,15 @@ sub call_procedure {
     # the binding process.  --Chris T
 
     foreach my $carg (@qargs){
-        if (ref($carg) eq ref {}){
+        if (ref($carg) =~ /HASH/){
             $sth->bind_param($place, $carg->{value},
                        { pg_type => $carg->{type} });
         } else {
 
             # This is used to support arrays of db-aware types.  Long-run 
             # I think we should merge bytea support into this framework. --CT
-            if (ref($carg) eq 'ARRAY'){
+            if (ref($carg) =~ /ARRAY/){
+               local ($@);
                if (eval{$carg->[0]->can('to_db')}){
                   for my $ref(@$carg){
                        $ref = $ref->to_db;
@@ -353,7 +358,7 @@ sub call_procedure {
         ++$place;
     }
 
-    $sth->execute();
+    $sth->execute() || die $dbh->errstr . ": " . $query;
     
     clear_info_cache() if $dbh->state eq '42883'; # (No Such Function)
 
@@ -385,10 +390,11 @@ This module should generally only be used by type handlers or by this module.
 
 sub process_type {
     my ($val, $type, $registry, $dbh) = @_;
+    return unless defined $val;
 
     $registry = $typeregistry{$registry} unless ref $registry;
     # Array handling as we'd get this usually from DBD::Pg or equivalent
-    if (ref $val eq ref []){
+    if (ref $val and ref($val) =~ /ARRAY/){
        # strangely, DBD::Pg returns, as of 2.x, the types of array types 
        # as prefixed with an underscore.  So we have to remove this. --CT
        $type =~ s/^\_//;
